package agents;

import sim.engine.SimState;
import sim.engine.Steppable;
import sim.engine.Stoppable;
import sim.portrayal.Portrayal;
import sim.util.Int2D;



public class Agent implements Steppable {
	public Stoppable event;
	
	public int patchID; 	//on which patch do I live?
	public int socialID;	//what is my social ID (of two)
	public double probMarker; //the probability that adoption is a social marker
	public int infected;

	public Int2D loc;
	public int timeSinceInfected = 0; //the number of timesteps since being infected
	
	public double influence1 = 0; //influence for product 1
	public double influence2 = 0; //influence for product 2
	public boolean awareOfProduct1 = false; 
	public boolean awareOfProduct2 = false; 
	
	public double memory = 0; 
	
	public double getinfluence(){return influence1;}
	
	
       /**
        * Constructor 
        */
       public Agent(int socid, int pid, Int2D agentLoc, AgentsSimulation as){
		infected = as.SUSCEPTIBLE;
		probMarker = 0.5;
		socialID = socid;
		patchID = pid;
		loc = agentLoc;
		influence1 = 0;
		influence2 = 0;
		memory = 0;
		awareOfProduct1 = false;
		awareOfProduct2 = false;
		this.event = as.schedule.scheduleRepeating(this, 1, 1);
       }
        
       public double getmemory(){return memory;}
    
       
   	/*********************** STEP METHOD ***************************************************************/
   	public synchronized void step(SimState state){
   		AgentsSimulation as = (AgentsSimulation)state;
   		//NEW CODE
   		if(as.random.nextBoolean(as.interactionRate)){//make an observation
   			double [] obs = makeObservation(as); //observe a sample of other agents
   			chooseAdoption(obs, as); //decide wether to 
   		}
   	}//end step
   	
   	
   	/*********************** OPERATIONAL METHODS ***************************************************************/
       
   	
   	/**
   	 * Observe a number of agents from within and/or outside the focal patch
   	 * returns a 2-item vector to use to calculate the probability of adoption.  
   	 */
   	public double[] makeObservation(AgentsSimulation as){
   		int numAdopters = 0;//total number of adopters
   		int numAdoptersIN = 0;//number of ingroup adopters
   		int numAdoptersOUT = 0;//number of outgroup adopters
   		final int numObs = as.observationSampleSize;
   		
   		for(int i = 0 ; i < numObs; i++){
   			Agent a = pickObservedAgent(as);
   			if(a.infected == as.INFECTED1){
   				numAdopters++;
   				if(a.socialID == this.socialID)
   					numAdoptersIN++;
   				else
   					numAdoptersOUT++;
   			}
   		}
   		double x = (double)numAdopters/(double)numObs;
   		double y = numAdoptersIN - numAdoptersOUT;
   		double[] obs = {x, y};
   		return obs;
   	}
   	
	
   	/**
	 * Choose an observed agent
	 */
	public Agent pickObservedAgent(AgentsSimulation as){
		//Is the observed agent from the same patch as the focal agent?
		int obs_PatchID = 0;
		if(as.random.nextBoolean(as.probSamePatch))
			obs_PatchID = this.patchID;
		else if (as.outPatchNeighbors){
			if(this.patchID == 0)
				obs_PatchID = 1;
			else if (this.patchID == as.numPatches - 1)
				obs_PatchID = as.numPatches - 2;
			else{
				if(as.random.nextBoolean(0.5))
					obs_PatchID = this.patchID + 1;
				else
					obs_PatchID = this.patchID - 1;
			}
			
		}
		else{
			do{
			obs_PatchID = as.random.nextInt(as.numPatches); //choose a random external patch
			}while(obs_PatchID == this.patchID); //make sure it's not home patch
		}
		
		int agentID = as.random.nextInt(as.numAgentsPerPatch); //choose a random agent in that patch
		Agent a = as.allPatches[obs_PatchID].patchAgents[agentID];
		
		while (a == this){ //make sure the agent doesn't observe itself
			agentID = as.random.nextInt(as.numAgentsPerPatch); 
			a = as.allPatches[obs_PatchID].patchAgents[agentID];
		}
		return a;
	}
	
	
	/**
	 * Decide whether to adopt the innovation
	 * obs[0] = x, for the F function. 
	 * obs[1] = mI - mO, for the B function
	 */
	public void chooseAdoption(double[] obs, AgentsSimulation as){
		double F = Math.pow(obs[0], as.lambda);
		
		double denom = 1 + Math.exp(-1 * as.ingroupRateOfIncrease * (obs[1] + memory));
		if(as.useMemory) //if using memory, let observations accumulate. 
			memory += obs[1];
		double B = 1 - as.ingroupBias + as.ingroupBias/denom;
		
		double I = 1; //For the future: this could be institutional incentives
		
		double probAdopt = F * B * I;
		
		if(as.random.nextBoolean(probAdopt)){
			infected = as.INFECTED1; //adopt product
		}
		else{
			infected = as.SUSCEPTIBLE; //don't adopt product
		}
		
		updateColor(as);
	}
	
	
	

   	
   	
	/**
	 * Update the agent's color 
	 */
	public void updateColor(AgentsSimulation as){
		Portrayal port = null;
		if(socialID == 0){
			if(infected == as.INFECTED1)
				port = as.type0_inf1;
			else
				port = as.type0_sus;
		}
		else{
			if(infected == as.INFECTED1)
				port = as.type1_inf1;
			else
				port = as.type1_sus;	
		}
		
		as.agentsPortrayal.setPortrayalForObject(this, port);
	}
   	
   	
	
}//end class
