package agents;
import sim.engine.*;
import sim.display.*;
import sim.portrayal.continuous.ContinuousPortrayal2D;
import sim.portrayal.grid.FastValueGridPortrayal2D;
import sim.portrayal.grid.ObjectGridPortrayal2D;
import sim.portrayal.grid.SparseGridPortrayal2D;
import sim.portrayal.network.NetworkPortrayal2D;
import sim.portrayal.network.SimpleEdgePortrayal2D;
import sim.portrayal.network.SpatialNetwork2D;
import sim.portrayal.simple.OvalPortrayal2D;
import sim.util.media.chart.*;	// the charting facility

import org.jfree.data.xy.*;

import sim.util.DoubleBag;
import java.awt.*;

import javax.swing.*;

//import agents.states.SimStateWithSimController;
import sim.portrayal.*;
import sim.util.*;

/*
 * The world is a large square that shows a number of smaller squares, each denoting a patch. 
 * Within this patch will be colored ovals or rectangles. The colors will change as agents adopt the innovation. 
 * 
 */

public class GraphicalUI extends GUIState{
    
    public static boolean updateVisuals = true;
    
    public Display2D display; 
    public JFrame displayFrame;
    static SimState ssc;
    public AgentsSimulation as;
    
    NetworkPortrayal2D edgePortrayal = new NetworkPortrayal2D(); //portrayal for the edges between patches
	SparseGridPortrayal2D nodePortrayal = new SparseGridPortrayal2D(); //perfunctory placehold for edges
	ObjectGridPortrayal2D agentsPortrayal = new ObjectGridPortrayal2D(); //for objects
	
	FastValueGridPortrayal2D patchPortrayal = new FastValueGridPortrayal2D("Patches");
	
	
	//make portrayals for the different agent kinds. 
	public Portrayal type0_sus = new sim.portrayal.simple.OvalPortrayal2D(Color.orange, (float)0.5);
	public Portrayal type0_inf1 = new sim.portrayal.simple.OvalPortrayal2D(Color.red, (float)0.8);
	public Portrayal type0_inf2 = new sim.portrayal.simple.OvalPortrayal2D(Color.blue, (float)0.8);
	
	public Portrayal type1_sus = new sim.portrayal.simple.RectanglePortrayal2D(Color.cyan, (float)0.5);
	public Portrayal type1_inf1 = new sim.portrayal.simple.RectanglePortrayal2D(Color.green, (float)0.8);
	public Portrayal type1_inf2 = new sim.portrayal.simple.RectanglePortrayal2D(Color.yellow, (float)0.8);
   
   /************* VARIABLES FOR CHARTING **************************/
    // Each series of points graphed in the chart must be created
    public XYSeries seriesFreqInfected_group1_productA; //the proportion of type 1 agents who adopt product A
    public XYSeries seriesFreqInfected_group1_productB; //the proportion of type 1 agents who adopt product A
    public XYSeries seriesFreqInfected_group2_productA; //the proportion of type 1 agents who adopt product A
    public XYSeries seriesFreqInfected_group2_productB; //the proportion of type 1 agents who adopt product A
    public TimeSeriesChartGenerator chart1;

   
    public GraphicalUI() {
		// builds model and portrayals
		super(new AgentsSimulation(System.currentTimeMillis()));
		//agentsPortrayal = new SparseGridPortrayal2D();
		as = (AgentsSimulation)state;
	}

	// standard construtor
    public GraphicalUI(SimState state) {
		super(state);
		as = (AgentsSimulation)state;
	}
    
    /******************* SET UP PORTRAYALS ********************************************************************/
    public void setupPortrayals() {
    	AgentsSimulation as = (AgentsSimulation)state;
    	
    	if(as.doGraphics){
    		agentsPortrayal.setField(as.agentsSpace); 
    		for(int x = 0; x < as.agentsSpace.getWidth(); x++)
    			for(int y = 0; y < as.agentsSpace.getHeight(); y++){
    				Agent a = (Agent)as.agentsSpace.get(x, y);
    				if(a.socialID == 0){
    					if(a.infected == as.SUSCEPTIBLE)
    						agentsPortrayal.setPortrayalForObject(a,as.type0_sus);
    					else if (a.infected == as.INFECTED1)
    						agentsPortrayal.setPortrayalForObject(a,as.type0_inf1);
    					else
    						agentsPortrayal.setPortrayalForObject(a,type0_inf2);
    				}
    				else{
    					if(a.infected == as.SUSCEPTIBLE)
    						agentsPortrayal.setPortrayalForObject(a,as.type1_sus);
    					else if (a.infected == as.INFECTED1)
    						agentsPortrayal.setPortrayalForObject(a,as.type1_inf1);
    					else
    						agentsPortrayal.setPortrayalForObject(a,type1_inf2);
    				}
    			}
    		as.agentsPortrayal = agentsPortrayal;  
    		
    		
    		edgePortrayal.setField( new SpatialNetwork2D( as.patchLines, as.patchBorders ) );
            edgePortrayal.setPortrayalForAll(new SimpleEdgePortrayal2D());
        	nodePortrayal.setField(as.patchLines);
        	nodePortrayal.setPortrayalForAll(new sim.portrayal.simple.RectanglePortrayal2D(new Color((float)0.1, (float)0.7, (float)0.2, (float)0.1), 0.1));
    		
        	patchPortrayal.setMap(new sim.util.gui.SimpleColorMap(0,1,Color.white,
        																new Color((float)0.6, (float)0.7, (float)0.6, (float)0.1)));
        	patchPortrayal.setField(as.patchSpace);
        	
        	
        	
    		display.reset(); 
    		display.repaint(); // call the repaint method
    	}
    }
    
    
	
	
    
    public void init(Controller c){
                
        super.init(c);  
        display = new Display2D(500,500,this,1); 
        displayFrame = display.createFrame(); 
        display.setClipping(false);
        c.registerFrame(displayFrame);   
        displayFrame.setVisible(true);  
        display.setBackdrop(Color.lightGray);
        if(((AgentsSimulation)state).doGraphics){
        	display.attach(patchPortrayal, "Patches");
        	display.attach(agentsPortrayal,"Agents");
        }
        
      /// CHART
        chart1 = new sim.util.media.chart.TimeSeriesChartGenerator();
        chart1.setYAxisLabel("Frequency of Adoption");
        chart1.setXAxisLabel("Time steps");
        chart1.setTitle("Adoption of Innovation");
        chart1.setYAxisRange(0, 1);

        JFrame frameC1 = chart1.createFrame(this);
        frameC1.setVisible(true);
        frameC1.pack();
        c.registerFrame(frameC1);

    }
    
    
        
    public void quit() {
         super.quit(); // Use the already defined quit method
            
         if (displayFrame!=null) displayFrame.dispose(); 
            displayFrame = null;  // when quiting get rid of the display
            display = null;       
    }
    
    public Object getSimulationInspectedObject() { return state; }
    
    public Inspector getInspector()
        {
        Inspector i = super.getInspector();
        i.setVolatile(true);
        return i;
        }

    
    /******************* START METHOD ********************************************************************/
    public void start() {
        super.start(); // use the predefined start method
        setupPortrayals(); // add setupPortrayals method below
         
        AgentsSimulation as = (AgentsSimulation) state;

        //For charting
        chart1.removeAllSeries();
        seriesFreqInfected_group1_productA = new org.jfree.data.xy.XYSeries("Group 1 adoption",false);
        seriesFreqInfected_group2_productA = new org.jfree.data.xy.XYSeries("Group 2 adoption",false);
        

        //chart1.addSeries(seriesFreqInfected_group1_productA, null);
        //chart1.addSeries(seriesFreqInfected_group1_productB, null);
        //chart1.addSeries(seriesFreqInfected_group2_productA, null);
        //chart1.addSeries(seriesFreqInfected_group2_productB, null);
        
        if(as.doGraphics) //only update if doing graphics (not for batch runs)
        scheduleRepeatingImmediatelyAfter(new Steppable()
            {
               public void step(SimState state)  {
            	   AgentsSimulation as = (AgentsSimulation) state;
            	   
            	   double x = state.schedule.getTime();//.time();
                   double y0 = AgentsSimulation.stepInfected_0A;
                   double y2 = AgentsSimulation.stepInfected_1A;
       
                   // now add the data
                   seriesFreqInfected_group1_productA.add(x, y0, true);
                   seriesFreqInfected_group2_productA.add(x, y2, true);
   
           }
           }); 
    }

        
    public void load(SimState state) {
        super.load(state); // load the simulation into the interface
        if (updateVisuals == true) setupPortrayals(); // call setuuPortrayals
    }
    
    
    public static void main(String[] args) {
     
        GraphicalUI gui = new GraphicalUI();
        Console c = new Console(gui);
        c.setVisible(true); // make the console visible
           
        System.out.println("Start Simulation"); 
    }
	

	
	

}//end class
