package tools;


import sim.util.Double2D;
import java.util.*;

public class Tools{
	
	
	/**
	 * Returns the Euclidean distance between (x1, y1) and (x2, y2).
	 */
	public static double distance(double x1, double y1, double x2, double y2){
		double x = Math.pow((x1 - x2), 2.0);
		double y = Math.pow((y1 - y2), 2.0);
		double p = Math.sqrt((x + y));
		return p;
	}

	/**
	 * Returns the Euclidean distance between (x1, y1) and (loc2.x, loc2.y).
	 */
	public static double distance(double x1, double y1, Double2D loc2){
		double x = Math.pow((x1 - loc2.x), 2.0);
		double y = Math.pow((y1 - loc2.y), 2.0);
		double p = Math.sqrt((x + y));
		return p;
	}

	/**
	 * Returns the Euclidean distance between (loc1.x, loc1.y) and (loc2.x, loc2.y).
	 */
	public static double distance(Double2D loc1, Double2D loc2){
		double x = Math.pow((loc1.x - loc2.x), 2.0);
		double y = Math.pow((loc1.y - loc2.y), 2.0);
		double p = Math.sqrt((x + y));
		return p;
	}

	/**
	 * Returns a Double2D that is the midpoint between two points.
	 */
	public static Double2D midpoint(Double2D loc1, Double2D loc2){
		double x = 0.5*(loc1.x + loc2.x);
		double y = 0.5*(loc1.y + loc2.y);
		return new Double2D(x, y);
	}
	
	
	
	/**
	 * truncates the double w to i significant digits
	 */
	public static double truncate(double w, int i){
		double p = w * Math.pow(10, i);
		p = Math.floor(p + 0.5);
		p = p/Math.pow(10, i);
		return p;
	}
	
	/**
	 * Takes an angle in radian measures and makes sure it falls into the
	 * boundary (-pi, pi). This is useful when using the atan function. 
	 */
	public static double convertAngle(double ang){
		if(ang > Math.PI)
			ang -= 2 * Math.PI;
		if(ang < -Math.PI)
			ang += 2 * Math.PI;
		
		return ang;
	}
	
	
	/**
	 * Returns the maximum value of a vector
	 */
	public static double maximum(double[] vector){ 
		double[] p = vector.clone();
		Arrays.sort(p);
		return p[p.length - 1];
	}
	
	/**
	 * Returns the maximum value of a vector
	 */
	public static int maximum(int[] vector){ 
		int[] p = vector.clone();
		Arrays.sort(p);
		return p[p.length - 1];
	}
	
	/**
	 * Returns the minimum value of a vector
	 */
	public static int minimum(int[] vector){ 
		int[] p = vector.clone();
		Arrays.sort(p);
		return p[0];
	}
	
	/**
	 * Returns the minimum value of a vector
	 */
	public static double minimum(double[] vector){ 
		double[] p = vector.clone();
		Arrays.sort(p);
		return p[0];
	}
	
	/**
	 * Returns the index of the maximum value of a vector
	 */
	public static int maxIndex(int[] vector){ 
		int maxVal = maximum(vector);
		int max = 0;
		int count = 0;
		for(int i = 0; i < vector.length; i++)
			if(vector[i] == maxVal){
				max = i;
				count++;
			}
		if(count == 1)//if there's only one minimum.
			return max;
		else{
			int[] maxes = new int[count];
			int index = 0;
			for(int i = 0; i < vector.length; i++)
				if(vector[i] == maxVal){
					maxes[index] = i;
					index++;
				}
			final int ind = (int)Math.floor(Math.random()*count);
			return maxes[ind];
		}
	}
	
	/**
	 * Returns the index of the maximum value of a vector
	 */
	public static int maxIndex(double[] vector){ 
		double maxVal = maximum(vector);
		int max = 0;
		int count = 0;
		for(int i = 0; i < vector.length; i++)
			if(vector[i] == maxVal){
				max = i;
				count++;
			}
		if(count == 1)//if there's only one minimum.
			return max;
		else{	//if more than one maximum 
			int[] maxes = new int[count];
			int index = 0;
			for(int i = 0; i < vector.length; i++)
				if(vector[i] == maxVal){
					maxes[index] = i;
					index++;
				}
			final int ind = (int)Math.floor(Math.random()*count);
			return maxes[ind];
		}
	}
	
	/**
	 * Returns the index of the minimum value of a vector
	 */
	public static int minIndex(int[] vector){ 
		int minVal = minimum(vector);
		int min = 0;
		int count = 0;
		for(int i = 0; i < vector.length; i++)
			if(vector[i] == minVal){
				min = i;
				count++;
			}
		if(count == 1)//if there's only one minimum.
			return min;
		else{
			int[] mins = new int[count];
			int index = 0;
			for(int i = 0; i < vector.length; i++)
				if(vector[i] == minVal){
					mins[index] = i;
					index++;
				}
			final int ind = (int)Math.floor(Math.random()*count);
			return mins[ind];
		}
	}
	
	/**
	 * Return the mean of a vector of doubles.
	 */
	public static double mean(double[] vector){
		double total = 0.0;
		double n = (double)vector.length;
		for(int i = 0; i < n; i++)
			total += vector[i];
		return total/n;
	}
	
	/**
	 * Return the sum of a vector of ints
	 */
	public static int sum(int[] vector){
		int sum = 0;
		for(int i = 0; i < vector.length; i++)
			sum += vector[i];
		return sum;
	}
	
	/**
     * calculates the standard deviation of an array of doubles.
     */
    public static double stdev(double[] vector){
      double avg = mean(vector);
  	  double val = 0.0;
  	  double sumSq = 0.0;
  	  for(int i = 0; i < vector.length; i++)
  		  sumSq += Math.pow((double)(vector[i] - avg), 2.0); //sum of the squares.
  	  val = (1/(double)(vector.length))*sumSq;
  	  val = Math.sqrt(val);
  	  return val;
    }
	
	/**
     * calculates the standard deviation of an array of doubles when the mean is already known. 
     */
    public static double stdev(double[] vector, double avg){
      //double avg = mean(vector);
  	  double val = 0.0;
  	  double sumSq = 0.0;
  	  for(int i = 0; i < vector.length; i++)
  		  sumSq += Math.pow((double)(vector[i] - avg), 2.0); //sum of the squares.
  	  val = (1/(double)(vector.length))*sumSq;
  	  val = Math.sqrt(val);
  	  return val;
    }
	
    
    
    /**
     * calculates the variance of an array of doubles.
     */
    public static double variance(double[] vector){
      double avg = mean(vector);
  	  double val = 0.0;
  	  double sumSq = 0.0;
  	  for(int i = 0; i < vector.length; i++)
  		  sumSq += Math.pow((double)(vector[i] - avg), 2.0); //sum of the squares.
  	  val = (1/(double)(vector.length))*sumSq;
  	  //val = Math.sqrt(val);
  	  return val;
    }
    
    /**
     *  calculates the variance of an 2-column array of doubles
     *  array (data) must be of the form double[2][n].
     */
    public static double covariance(double[][] data){
    	double mean1 = mean(data[0]);
    	double mean2 = mean(data[1]);
    	int length = data[0].length;
    	double sum = 0;
    	for(int i = 0; i < length; i++)
    		sum += data[0][i] * data[1][i];
    	double xy = sum/(double)length;
    	
    	return xy - (mean1 * mean2);
    }
	
	/**
	 * Return the median of a vector of doubles.
	 */
	public static double median(double[] vector){
		double[] p = vector.clone();
		Arrays.sort(p);
		int middle = p.length/2;
		if(p.length%2 == 1)
			return p[middle];
		else
			return (p[middle-1] + p[middle])/2.0;
	}
	
	/**
	 * Return the skew of a vector of doubles when the mean and sd is already known
	 */
	public static double skew(double[] vector, double avg, double sd){
		//double avg = mean(vector);
	    double sum = 0.0;	    
	    for(int i = 0; i < vector.length; i++)
	    	sum += Math.pow((vector[i] - avg), 3.0); //sum of the squares.
	  
	    double val = sum/(double)(vector.length);
	  	return val/(Math.pow(sd, 3.0));
	}
	
	/**
	 * Return the kurtosis of a vector of doubles when the mean and sd is already known
	 */
	public static double kurtosis(double[] vector, double avg, double sd){
		//double avg = mean(vector);
	    double sum = 0.0;	    
	    for(int i = 0; i < vector.length; i++)
	    	sum += Math.pow((vector[i] - avg), 4.0); //sum of the squares.
	  
	    double val = sum/(double)(vector.length);
	  	return val/(Math.pow(sd, 4.0));
	}
	
}//end class